@@ -1,6 +1,5 @@
-//
 // Basemenu.cc for Blackbox - an X11 Window manager
-// Copyright (c) 1997, 1998 by Brad Hughes, bhughes@tcac.net
+// Copyright (c) 1997 - 1999 by Brad Hughes, bhughes@tcac.net
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,25 +18,26 @@
 // (See the included file COPYING / GPL-2.0)
 //
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
+#ifndef   _GNU_SOURCE
+#define   _GNU_SOURCE
+#endif // _GNU_SOURCE
 
-#ifdef HAVE_CONFIG_H
+#ifdef    HAVE_CONFIG_H
 #  include "../config.h"
-#endif
+#endif // HAVE_CONFIG_H
 
 #include "blackbox.hh"
 #include "Basemenu.hh"
 #include "Screen.hh"
 
-#if HAVE_STDIO_H
+#ifdef    HAVE_STDIO_H
 #  include <stdio.h>
-#endif
+#endif // HAVE_STDIO_H
 
-#if STDC_HEADERS
+#ifdef    STDC_HEADERS
 #  include <stdlib.h>
-#endif
+#  include <string.h>
+#endif // STDC_HEADERS
 
 
 static Basemenu *shown = (Basemenu *) 0;
@@ -48,13 +48,15 @@ Basemenu::Basemenu(Blackbox *ctrl, BScreen *scrn) {
   image_ctrl = screen->getImageControl();
   display = blackbox->getDisplay();
   parent = (Basemenu *) 0;
-  alignment = MenuAlignDontCare;
+  alignment = AlignDontCare;
 
   title_vis = movable = hidable = True;
   shifted = default_menu = moving = user_moved = visible = False;
   menu.x = menu.y = menu.x_shift = menu.y_shift = menu.x_move =
     menu.y_move = 0;
   always_highlight = which_sub = which_press = which_sbl = -1;
+  indicator = screen->getBulletStyle();
+  indicator_position = screen->getBulletPosition();
 
   menu.iframe_pixmap = menu.title_pixmap = None;
 
@@ -63,31 +65,30 @@ Basemenu::Basemenu(Blackbox *ctrl, BScreen *scrn) {
     screen->getTitleFont()->ascent + screen->getTitleFont()->descent +
     (menu.bevel_w * 2);
   menu.label = 0;
-  menu.sublevels = menu.persub = 0;
+  menu.sublevels = menu.persub = menu.minsub = 0;
   menu.item_h = screen->getMenuFont()->ascent +
     screen->getMenuFont()->descent + (menu.bevel_w);
-  menu.height = menu.title_h + 1 + menu.iframe_h;
+  menu.height = menu.title_h + screen->getBorderWidth() + menu.iframe_h;
   
-  unsigned long attrib_mask = CWBackPixmap|CWBackPixel|CWBorderPixel|
-    CWOverrideRedirect|CWCursor|CWEventMask;
+  unsigned long attrib_mask = CWBackPixmap | CWBackPixel | CWBorderPixel |
+    CWOverrideRedirect | CWEventMask;
   XSetWindowAttributes attrib;
   attrib.background_pixmap = None;
   attrib.background_pixel = attrib.border_pixel =
-    screen->getBorderColor().pixel;
+    screen->getBorderColor()->getPixel();
   attrib.override_redirect = True;
-  attrib.cursor = blackbox->getSessionCursor();
   attrib.event_mask = ButtonPressMask | ButtonReleaseMask | ButtonMotionMask |
     ExposureMask;
 
   menu.frame =
     XCreateWindow(display, screen->getRootWindow(), menu.x, menu.y, menu.width,
-		  menu.height, 1, screen->getDepth(), InputOutput,
-		  screen->getVisual(), attrib_mask, &attrib);
+		  menu.height, screen->getBorderWidth(), screen->getDepth(),
+                  InputOutput, screen->getVisual(), attrib_mask, &attrib);
   blackbox->saveMenuSearch(menu.frame, this);
   
-  attrib_mask = CWBackPixmap|CWBackPixel|CWBorderPixel|CWCursor|CWEventMask;
-  attrib.background_pixel = screen->getBorderColor().pixel;
-  attrib.event_mask |= EnterWindowMask|LeaveWindowMask;
+  attrib_mask = CWBackPixmap | CWBackPixel | CWBorderPixel | CWEventMask;
+  attrib.background_pixel = screen->getBorderColor()->getPixel();
+  attrib.event_mask |= EnterWindowMask | LeaveWindowMask;
 
   menu.title =
     XCreateWindow(display, menu.frame, 0, 0, menu.width, menu.height, 0,
@@ -95,7 +96,8 @@ Basemenu::Basemenu(Blackbox *ctrl, BScreen *scrn) {
 		  attrib_mask, &attrib);
   blackbox->saveMenuSearch(menu.title, this);
   
-  menu.iframe = XCreateWindow(display, menu.frame, 0, menu.title_h + 1,
+  menu.iframe = XCreateWindow(display, menu.frame, 0,
+                              menu.title_h + screen->getBorderWidth(),
 			      menu.width, menu.iframe_h, 0,
 			      screen->getDepth(), InputOutput,
 			      screen->getVisual(), attrib_mask, &attrib);
@@ -143,7 +145,22 @@ Basemenu::~Basemenu(void) {
 }
 
 
-int Basemenu::insert(char *label, int function, char *exec, int pos) {
+int Basemenu::insert(char *l, int function, char *e, int pos) {
+  char *label = 0, *exec = 0;
+  int llen, elen;
+  
+  if (l) {
+    llen = strlen(l) + 1;
+    label = new char[llen];
+    strncpy(label, l, llen);
+  }
+  
+  if (e) {
+    elen = strlen(e) + 1;
+    exec = new char[elen + 1];
+    strncpy(exec, e, elen + 1);
+  }
+  
   BasemenuItem *item = new BasemenuItem(label, function, exec);
   menuitems->insert(item, pos);
   
@@ -151,13 +168,22 @@ int Basemenu::insert(char *label, int function, char *exec, int pos) {
 }
 
 
-int Basemenu::insert(char *label, Basemenu *submenu, int pos) {
+int Basemenu::insert(char *l, Basemenu *submenu, int pos) {
+  char *label = 0;
+  int llen = strlen(l);
+
+  if (llen) {
+    label = new char[llen + 1];
+    strncpy(label, l, llen + 1);
+  }
+  
   BasemenuItem *item = new BasemenuItem(label, submenu);
   menuitems->insert(item, pos);
+  
   submenu->parent = this;
-  submenu->setMovable(True);
-  submenu->setHidable(False);
-
+  if (submenu->title_vis) submenu->setMovable(True);
+  if (! submenu->default_menu) submenu->setHidable(False);
+  
   return menuitems->count();
 }
 
@@ -165,34 +191,41 @@ int Basemenu::insert(char *label, Basemenu *submenu, int pos) {
 int Basemenu::insert(char **ulabel, int pos) {
   BasemenuItem *item = new BasemenuItem(ulabel);
   menuitems->insert(item, pos);
-
+  
   return menuitems->count();
 }
 
 
 int Basemenu::remove(int index) {
+  if (index < 0 || index > menuitems->count()) return -1;
+
   BasemenuItem *item = menuitems->remove(index);
 
-  if (! default_menu) {
-    if (item->submenu()) {
+  if (item) {
+    if ((! default_menu) && (item->submenu()) &&
+	(! item->submenu()->default_menu)) {
       Basemenu *tmp = (Basemenu *) item->submenu();
       delete tmp;
     }
-    if (item->label()) {
+    
+    if (item->label())
       delete [] item->label();
-    }
-    if (item->exec()) {
+    
+    if (item->exec())
       delete [] item->exec();
-    }
+    
+    delete item;
   }
-  
-  delete item;
 
   if (which_sub == index)
     which_sub = -1;
-  
+  else if (which_sub > index)
+    which_sub--;
+
   if (always_highlight == index)
     always_highlight = -1;
+  else if (always_highlight > index)
+    always_highlight--;
 
   return menuitems->count();
 }
@@ -214,6 +247,7 @@ void Basemenu::update(void) {
   LinkedListIterator<BasemenuItem> it(menuitems);
   for (; it.current(); it++) {
     BasemenuItem *itmp = it.current();
+
     if (itmp->u)
       ii = (menu.bevel_w * 2) + (menu.item_h * 2) +
 	XTextWidth(screen->getMenuFont(), *itmp->u, strlen(*itmp->u));
@@ -228,13 +262,16 @@ void Basemenu::update(void) {
   
   if (menuitems->count()) {
     menu.sublevels = 1;
-    while ((((menu.item_h + 1) * (menuitems->count() + 1) / menu.sublevels)
-	    + menu.title_h + 1) >
-	   screen->getYRes())
+
+    while (((menu.item_h * (menuitems->count() + 1) / menu.sublevels)
+	    + menu.title_h + screen->getBorderWidth()) >
+	   screen->getHeight())
       menu.sublevels++;
+
+    if (menu.sublevels < menu.minsub) menu.sublevels = menu.minsub;
     
     menu.persub = menuitems->count() / menu.sublevels;
-    if (menuitems->count() % menu.sublevels) menu.sublevels++;
+    if (menuitems->count() % menu.sublevels) menu.persub++;
   } else {
     menu.sublevels = 0;
     menu.persub = 0;
@@ -246,8 +283,9 @@ void Basemenu::update(void) {
   menu.title_h = screen->getTitleFont()->ascent +
     screen->getTitleFont()->descent + (menu.bevel_w * 2);
 
-  menu.iframe_h = ((menu.item_h + 1) * menu.persub);
-  menu.height = ((title_vis) ? menu.title_h + 1 : 0) + menu.iframe_h;
+  menu.iframe_h = (menu.item_h * menu.persub);
+  menu.height = ((title_vis) ? menu.title_h + screen->getBorderWidth() : 0) +
+    menu.iframe_h;
   if (! menu.iframe_h) menu.iframe_h = 1;
   if (menu.height < 1) menu.height = 1;
 
@@ -275,7 +313,8 @@ void Basemenu::update(void) {
     XResizeWindow(display, menu.title, menu.width, menu.title_h);
   
   XMoveResizeWindow(display, menu.iframe, 0,
-		    ((title_vis) ? menu.title_h + 1 : 0), menu.width,
+		    ((title_vis) ? menu.title_h +
+                                   screen->getBorderWidth() : 0), menu.width,
 		    menu.iframe_h);
   
   XClearWindow(display, menu.frame);
@@ -284,7 +323,7 @@ void Basemenu::update(void) {
   
   if (title_vis)
     switch (screen->getJustification()) {
-    case Blackbox::B_LeftJustify:
+    case BScreen::LeftJustify:
       {
 	XDrawString(display, menu.title, screen->getMenuTitleGC(),
 		    menu.bevel_w, screen->getTitleFont()->ascent +
@@ -294,7 +333,7 @@ void Basemenu::update(void) {
 	break;
       }
       
-    case Blackbox::B_RightJustify:
+    case BScreen::RightJustify:
       {
 	int off = menu.bevel_w +
 	  XTextWidth(screen->getTitleFont(),
@@ -309,7 +348,7 @@ void Basemenu::update(void) {
 	break;
       }
       
-    case Blackbox::B_CenterJustify:
+    case BScreen::CenterJustify:
       {
 	int ins = (menu.width -
 		   (XTextWidth(screen->getTitleFont(),
@@ -365,7 +404,8 @@ void Basemenu::hide(void) {
     parent->drawItem(parent->which_sub,
 		     (parent->always_highlight == parent->which_sub),
 		     False, True);
-    hidable = False;
+
+    if (! default_menu) hidable = False;
     parent->which_sub = -1;
   } else if (shown && shown->getWindowID() == getWindowID())
     shown = (Basemenu *) 0;
@@ -383,9 +423,10 @@ void Basemenu::move(int x, int y) {
 }
 
 
-void Basemenu::drawSubmenu(int index, Bool) {
+void Basemenu::drawSubmenu(int index) {
   if (which_sub != -1 && which_sub != index) {
     BasemenuItem *itmp = menuitems->find(which_sub);
+
     if (! itmp->submenu()->hasUserMoved())
       itmp->submenu()->hide();
   }
@@ -397,32 +438,34 @@ void Basemenu::drawSubmenu(int index, Bool) {
 
       int sbl = index / menu.persub, i = index - (sbl * menu.persub),
 	x = (((shifted) ? menu.x_shift : menu.x) +
-	     (menu.item_w * (sbl + 1)) + 1), y;
+	     (menu.item_w * (sbl + 1)) + screen->getBorderWidth()), y;
       
-      if (alignment == MenuAlignTop)
+      if (alignment == AlignTop)
 	y = (((shifted) ? menu.y_shift : menu.y) +
-	     ((title_vis) ? menu.title_h + 1 : 0) -
+	     ((title_vis) ? menu.title_h + screen->getBorderWidth() : 0) -
 	     ((item->submenu()->title_vis) ?
-	      item->submenu()->menu.title_h + 1 : 0));
+	      item->submenu()->menu.title_h + screen->getBorderWidth() : 0));
       else
 	y = (((shifted) ? menu.y_shift : menu.y) +
-	     ((menu.item_h + 1) * i) + ((title_vis) ? menu.title_h + 1 : 0) -
+	     (menu.item_h * i) +
+             ((title_vis) ? menu.title_h + screen->getBorderWidth() : 0) -
 	     ((item->submenu()->title_vis) ?
-	      item->submenu()->menu.title_h + 1 : 0));
+	      item->submenu()->menu.title_h + screen->getBorderWidth() : 0));
 
-      if (alignment == MenuAlignBottom &&
+      if (alignment == AlignBottom &&
 	  (y + item->submenu()->menu.height) > ((shifted) ? menu.y_shift :
 						menu.y) + menu.height)
 	y = (((shifted) ? menu.y_shift : menu.y) +
 	     menu.height - item->submenu()->menu.height);
       
-      if ((x + item->submenu()->getWidth()) > screen->getXRes())
+      if ((x + item->submenu()->getWidth()) > screen->getWidth())
 	x = ((shifted) ? menu.x_shift : menu.x) -
-	  item->submenu()->getWidth() - 1;
+	  item->submenu()->getWidth() - screen->getBorderWidth();
       if (x < 0) x = 0;
 
-      if ((y + item->submenu()->getHeight()) > screen->getYRes())
-	y = screen->getYRes() - item->submenu()->getHeight() - 1;
+      if ((y + item->submenu()->getHeight()) > screen->getHeight())
+	y = screen->getHeight() - item->submenu()->getHeight() -
+          screen->getBorderWidth();
       if (y < 0) y = 0;
       
       item->submenu()->move(x, y);
@@ -451,17 +494,17 @@ void Basemenu::drawItem(int index, Bool frame, Bool highlight, Bool clear) {
   if (index < 0 || index > menuitems->count()) return;
   
   int sbl = index / menu.persub, i = index - (sbl * menu.persub),
-    ix = (sbl * menu.item_w), iy = (i * (menu.item_h + 1)), tx = 0;
+    ix = (sbl * menu.item_w), iy = (i * menu.item_h), tx = 0;
   BasemenuItem *item = menuitems->find(index);
   
   if (! item) return;
   
   switch(screen->getMenuJustification()) {
-  case Blackbox::B_LeftJustify:
+  case BScreen::LeftJustify:
     tx = ix + menu.bevel_w + menu.item_h + 1;
     break;
 
-  case Blackbox::B_RightJustify:
+  case BScreen::RightJustify:
     tx = ix + menu.item_w -
       (menu.item_h + menu.bevel_w +
        XTextWidth(screen->getMenuFont(),
@@ -469,7 +512,7 @@ void Basemenu::drawItem(int index, Bool frame, Bool highlight, Bool clear) {
 		  strlen((item->ulabel()) ? *item->ulabel() : item->label())));
     break;
 
-  case Blackbox::B_CenterJustify:
+  case BScreen::CenterJustify:
     tx = ix + ((menu.item_w + 1 -
 		XTextWidth(screen->getMenuFont(),
 			   ((item->ulabel()) ? *item->ulabel() :
@@ -480,61 +523,106 @@ void Basemenu::drawItem(int index, Bool frame, Bool highlight, Bool clear) {
   }
   
   if (clear) {
-    XClearArea(display, menu.iframe, ix, iy, menu.item_w, menu.item_h + 1,
+    XClearArea(display, menu.iframe, ix, iy, menu.item_w, menu.item_h,
 	       False);
   }
 
-  if (frame)
-    XFillArc(display, menu.iframe, screen->getMenuHiBGGC(), ix + 1, iy + 1,
-             menu.item_h - 1, menu.item_h - 2, 0, 360 * 64);
-  
+  GC gc = (highlight || frame) ? screen->getMenuHiGC() :
+       screen->getMenuFrameGC(),
+     tgc = (highlight) ? screen->getMenuHiGC() : screen->getMenuFrameGC();
+
+  int fx = ix, hw = menu.item_h / 2, qw = menu.item_h / 4;
+    
+  if (indicator_position == Right)
+    fx += (menu.item_w - menu.item_h - menu.bevel_w);
+
   if (highlight) {
-    XFillArc(display, menu.iframe, screen->getMenuHiBGGC(), ix + 1, iy + 1,
-             menu.item_h - 1, menu.item_h - 2, 90 * 64, 180 * 64);
-    XFillArc(display, menu.iframe, screen->getMenuHiBGGC(), ix + menu.item_w -
-             menu.item_h - 2, iy + 1, menu.item_h - 1, menu.item_h - 2,
-             270 * 64, 180 * 64);
-    XFillRectangle(display, menu.iframe, screen->getMenuHiBGGC(), ix +
-                   ((menu.item_h + 1) / 2), iy + 1, menu.item_w -
-                   menu.item_h - 2, menu.item_h - 1);
+    if (indicator == Round) {
+      XFillArc(display, menu.iframe, screen->getMenuHiBGGC(), ix + 1, iy + 1,
+               menu.item_h - 1, menu.item_h - 2, 90 * 64, 180 * 64);
+      XFillArc(display, menu.iframe, screen->getMenuHiBGGC(),
+               ix + menu.item_w - menu.item_h, iy + 1, menu.item_h - 1,
+               menu.item_h - 2, 270 * 64, 180 * 64);
+      XFillRectangle(display, menu.iframe, screen->getMenuHiBGGC(), ix +
+                     (menu.item_h / 2), iy + 1, menu.item_w - menu.item_h,
+                     menu.item_h - 1);    
+    } else
+      XFillRectangle(display, menu.iframe, screen->getMenuHiBGGC(),
+                     ix + 1, iy + 1, menu.item_w - 2, menu.item_h - 2);
+   
+  } else if (frame) {
+    if (indicator == Round)
+      XFillArc(display, menu.iframe, screen->getMenuHiBGGC(), fx + 1, iy + 1,
+               menu.item_h - 1, menu.item_h - 2, 0, 360 * 64);
+    else
+      XFillRectangle(display, menu.iframe, screen->getMenuHiBGGC(),
+                     fx + qw, iy + qw, hw + 1, hw + 1);
+  }
+  
+  if (item->ulabel())
+    XDrawString(display, menu.iframe, tgc, tx,
+                iy + screen->getMenuFont()->ascent + (menu.bevel_w / 2) + 1,
+                *item->ulabel(), strlen(*item->ulabel()));
+  else if (item->label())
+    XDrawString(display, menu.iframe, tgc, tx,
+                iy + screen->getMenuFont()->ascent + (menu.bevel_w / 2) + 1,
+                item->label(), strlen(item->label()));
     
-    if (item->ulabel())
-      XDrawString(display, menu.iframe, screen->getMenuHiGC(), tx, iy +
-		  screen->getMenuFont()->ascent + (menu.bevel_w / 2) + 1,
-                  *item->ulabel(), strlen(*item->ulabel()));
-    else if (item->label())
-      XDrawString(display, menu.iframe, screen->getMenuHiGC(), tx, iy +
-		  screen->getMenuFont()->ascent + (menu.bevel_w / 2) + 1,
-                  item->label(), strlen(item->label()));
-    
-    if (item->submenu())
-      XDrawArc(display, menu.iframe, screen->getMenuHiGC(), ix +
-               (menu.item_h / 4) + 1, iy + (menu.item_h / 4) + 1,
-               menu.item_h / 2, menu.item_h / 2, 0, 360 * 64);
-  } else {
-    if (item->ulabel())
-      XDrawString(display, menu.iframe, screen->getMenuFrameGC(), tx, iy +
-		  screen->getMenuFont()->ascent + (menu.bevel_w / 2) + 1,
-                  *item->ulabel(), strlen(*item->ulabel()));
-    else if (item->label())
-      XDrawString(display, menu.iframe, screen->getMenuFrameGC(), tx, iy +
-		  screen->getMenuFont()->ascent + (menu.bevel_w / 2) + 1,
-                  item->label(), strlen(item->label()));
-    
-    if (item->submenu())
-      XDrawArc(display, menu.iframe, screen->getMenuFrameGC(), ix +
-               (menu.item_h / 4) + 1, iy + (menu.item_h / 4) + 1,
-               menu.item_h / 2, menu.item_h / 2, 0, 360 * 64);
+  if (item->submenu()) {
+    switch (indicator) {
+    case Empty:
+      break;
+
+    case Round:
+    default:
+      XDrawArc(display, menu.iframe, gc, fx + qw + 1, iy + qw + 1,
+               hw, hw, 0, 360 * 64);
+      break;
+
+    case Square:
+      XDrawRectangle(display, menu.iframe, gc, fx + qw, iy + qw, hw, hw);
+      break;
+                                  
+    case Triangle:
+      XPoint tri[3]; 
+
+      if (indicator_position == Right) {
+        tri[0].x = fx + hw - 2; tri[0].y = iy + hw - 2;
+        tri[1].x = 4; tri[1].y = 2;
+        tri[2].x = -4; tri[2].y = 2;
+      } else {
+        tri[0].x = fx + hw - 2; tri[0].y = iy + hw;
+        tri[1].x = 4; tri[1].y = 2; 
+        tri[2].x = 0; tri[2].y = -4;
+      }
+
+      XFillPolygon(display, menu.iframe, gc, tri, 3, Convex,
+                   CoordModePrevious);
+      break;
+
+    case Diamond:
+      XPoint dia[4];
+
+      dia[0].x = fx + hw - 3; dia[0].y = iy + hw;
+      dia[1].x = 3; dia[1].y = -3;
+      dia[2].x = 3; dia[2].y = 3;
+      dia[3].x = -3; dia[3].y = 3;
+
+      XFillPolygon(display, menu.iframe, gc, dia, 4, Convex,
+                   CoordModePrevious);
+      break;
+    }
   }
 }
 
 
-void Basemenu::setMenuLabel(char *l) {
+void Basemenu::setLabel(char *l) {
   if (menu.label) delete [] menu.label;
 
   if (l) {
-    menu.label = new char[strlen(l) + 1];
-    sprintf(menu.label, "%s", l);
+    int mlen = strlen(l) + 1;
+    menu.label = new char[mlen];
+    strncpy(menu.label, l, mlen);
   } else
     menu.label = 0;
 }
@@ -542,23 +630,19 @@ void Basemenu::setMenuLabel(char *l) {
 
 void Basemenu::setHighlight(int index) {
   if (always_highlight != -1)
-    drawItem(always_highlight, False, False, True);
+    drawItem(always_highlight, False, (which_sub == always_highlight), True);
   
   if (index >= 0 && index < menuitems->count()) {
     always_highlight = index; 
-    drawItem(always_highlight, True);
+    drawItem(always_highlight, True, (which_sub == always_highlight), True);
   } else
     always_highlight = -1;
 }
 
 
 void Basemenu::buttonPressEvent(XButtonEvent *be) {
-  if (be->window == menu.title) {
-    if (be->x >= 0 && be->x <= (signed) menu.width &&
-	be->y >= 0 && be->y <= (signed) menu.title_h) {
-    }
-  } else if (be->window == menu.iframe) {
-    int sbl = (be->x / menu.item_w), i = (be->y / (menu.item_h + 1));
+  if (be->window == menu.iframe) {
+    int sbl = (be->x / menu.item_w), i = (be->y / menu.item_h);
     int w = (sbl * menu.persub) + i;
 
     if (w < menuitems->count() && w >= 0) {
@@ -572,7 +656,6 @@ void Basemenu::buttonPressEvent(XButtonEvent *be) {
       else
 	drawItem(w, False, True);
     }
-    
   }
 }
 
@@ -596,8 +679,8 @@ void Basemenu::buttonReleaseEvent(XButtonEvent *re) {
     if (re->button == 3 && hidable) {
       hide();
     } else {
-      int sbl = (re->x / menu.item_w), i = (re->y / (menu.item_h + 1)),
-        ix = sbl * menu.item_w, iy = i * (menu.item_h + 1),
+      int sbl = (re->x / menu.item_w), i = (re->y / menu.item_h),
+        ix = sbl * menu.item_w, iy = i * menu.item_h,
         w = (sbl * menu.persub) + i,
         p = (which_sbl * menu.persub) + which_press;
 
@@ -654,7 +737,7 @@ void Basemenu::motionNotifyEvent(XMotionEvent *me) {
   } else if ((! (me->state & Button1Mask)) && me->window == menu.iframe &&
 	     me->x >= 0 && me->x < (signed) menu.width &&
 	     me->y >= 0 && me->y < (signed) menu.iframe_h) {
-    int sbl = (me->x / menu.item_w), i = (me->y / (menu.item_h + 1)),
+    int sbl = (me->x / menu.item_w), i = (me->y / menu.item_h),
       w = (sbl * menu.persub) + i;
     
     if ((i != which_press || sbl != which_sbl) &&
@@ -689,7 +772,7 @@ void Basemenu::motionNotifyEvent(XMotionEvent *me) {
 void Basemenu::exposeEvent(XExposeEvent *ee) {
   if (ee->window == menu.title) {
     switch (screen->getJustification()) {
-    case Blackbox::B_LeftJustify:
+    case BScreen::LeftJustify:
       {
 	XDrawString(display, menu.title, screen->getMenuTitleGC(),
 		    menu.bevel_w, screen->getTitleFont()->ascent +
@@ -699,7 +782,7 @@ void Basemenu::exposeEvent(XExposeEvent *ee) {
 	break;
       }
 
-    case Blackbox::B_RightJustify:
+    case BScreen::RightJustify:
       {
 	int off = XTextWidth(screen->getTitleFont(),
 			     ((menu.label) ? menu.label : "Blackbox Menu"),
@@ -713,7 +796,7 @@ void Basemenu::exposeEvent(XExposeEvent *ee) {
 	break;
       }
       
-    case Blackbox::B_CenterJustify:
+    case BScreen::CenterJustify:
       {
 	int ins = (menu.width -
 		   (XTextWidth(screen->getTitleFont(),
@@ -734,11 +817,11 @@ void Basemenu::exposeEvent(XExposeEvent *ee) {
     // first... we see in which sub level the expose starts... and how many
     // items down in that sublevel
     
-    int sbl = (ee->x / menu.item_w), id = (ee->y / (menu.item_h + 1)),
+    int sbl = (ee->x / menu.item_w), id = (ee->y / menu.item_h),
       // next... figure out how many sublevels over the redraw spans
       sbl_d = ((ee->x + ee->width) / menu.item_w),
       // then we see how many items down to redraw
-      id_d = ((ee->y + ee->height) / (menu.item_h + 1));
+      id_d = ((ee->y + ee->height) / menu.item_h);
     
     if (id_d > menu.persub) id_d = menu.persub;
     
@@ -765,23 +848,30 @@ void Basemenu::enterNotifyEvent(XCrossingEvent *ce) {
 		 blackbox->getSessionCursor(), CurrentTime);
     
     menu.x_shift = menu.x, menu.y_shift = menu.y;
-    if (menu.x + menu.width > screen->getXRes()) {
-      menu.x_shift = screen->getXRes() - menu.width - 1;
+    if (menu.x + menu.width > screen->getWidth()) {
+      menu.x_shift = screen->getWidth() - menu.width - screen->getBorderWidth();
+      shifted = True;
+    } else if (menu.x < 0) {
+      menu.x_shift = 0;
       shifted = True;
     }
     
-    if (menu.y + menu.height > screen->getYRes()) {
-      menu.y_shift = screen->getYRes() - menu.height - 1;
+    if (menu.y + menu.height > screen->getHeight()) {
+      menu.y_shift = screen->getHeight() - menu.height -
+        screen->getBorderWidth();
+      shifted = True;
+    } else if (menu.y + (signed) menu.title_h < 0) {
+      menu.y_shift = 0;
       shifted = True;
     }
-
+    
     if (shifted)
       XMoveWindow(display, menu.frame, menu.x_shift, menu.y_shift);
-
+    
     if (which_sub != -1) {
       BasemenuItem *tmp = menuitems->find(which_sub);
       if (tmp->submenu()->isVisible()) {
-	int sbl = (ce->x / menu.item_w), i = (ce->y / (menu.item_h + 1)),
+	int sbl = (ce->x / menu.item_w), i = (ce->y / menu.item_h),
 	  w = (sbl * menu.persub) + i;
 	
 	if (w != which_sub && (! tmp->submenu()->hasUserMoved())) {
@@ -807,21 +897,26 @@ void Basemenu::leaveNotifyEvent(XCrossingEvent *ce) {
       which_sbl = which_press = -1;
     }
     
-    if (! (ce->state & Button1Mask))
-      XUngrabPointer(display, CurrentTime);
-    
     if (shifted) {
       XMoveWindow(display, menu.frame, menu.x, menu.y);
       shifted = False;
     }
+
+    if (! (ce->state & Button1Mask))
+      XUngrabPointer(display, CurrentTime);
   }
 }
 
 
 void Basemenu::reconfigure(void) {
-  XSetWindowBackground(display, menu.frame, screen->getBorderColor().pixel);
-  XSetWindowBorder(display, menu.frame, screen->getBorderColor().pixel);
+  XSetWindowBackground(display, menu.frame,
+		       screen->getBorderColor()->getPixel());
+  XSetWindowBorder(display, menu.frame,
+		   screen->getBorderColor()->getPixel());
+  XSetWindowBorderWidth(display, menu.frame, screen->getBorderWidth());
 
   menu.bevel_w = screen->getBevelWidth();
+  indicator_position = screen->getBulletPosition();
+  indicator = screen->getBulletStyle();
   update();
 }
