@@ -1,6 +1,5 @@
-//
-// Basemenu.cc for Blackbox - an X11 Window manager
-// Copyright (c) 1997, 1998 by Brad Hughes, bhughes@tcac.net
+// Rootmenu.cc for Blackbox - an X11 Window manager
+// Copyright (c) 1997 - 1999 by Brad Hughes, bhughes@tcac.net
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,30 +18,30 @@
 // (See the included file COPYING / GPL-2.0)
 //
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
+#ifndef   _GNU_SOURCE
+#define   _GNU_SOURCE
+#endif // _GNU_SOURCE
 
-#ifdef HAVE_CONFIG_H
+#ifdef    HAVE_CONFIG_H
 #  include "../config.h"
-#endif
+#endif // HAVE_CONFIG_H
 
 #include "blackbox.hh"
 #include "Rootmenu.hh"
 #include "Screen.hh"
 
-#if HAVE_STDIO_H
+#ifdef    HAVE_STDIO_H
 #  include <stdio.h>
-#endif
+#endif // HAVE_STDIO_H
 
-#if STDC_HEADERS
+#ifdef    STDC_HEADERS
 #  include <stdlib.h>
 #  include <string.h>
-#endif
+#endif // STDC_HEADERS
 
-#if HAVE_PROCESS_H && __EMX__
+#if defined(HAVE_PROCESS_H) && defined(__EMX__)
 #  include <process.h>
-#endif
+#endif //   HAVE_PROCESS_H             __EMX__
 
 
 Rootmenu::Rootmenu(Blackbox *bb, BScreen *scrn) : Basemenu(bb, scrn) {
@@ -57,58 +56,51 @@ void Rootmenu::itemSelected(int button, int index) {
 
     if (item->function()) {
       switch (item->function()) {
-      case Blackbox::B_Execute:
+      case BScreen::Execute:
 	if (item->exec()) {
-#ifndef __EMX__
-          int dslen = strlen(DisplayString(screen->getDisplay()));
+#ifndef   __EMX__
+          int dslen =
+            strlen(DisplayString(screen->getDisplay()->getDisplay()));
 	  
           char *displaystring = new char[dslen + 32];
           char *command = new char[strlen(item->exec()) + dslen + 64];
 	  
-          strncpy(displaystring, DisplayString(screen->getDisplay()),
-                  dslen - 1);
+          sprintf(displaystring, "%s",
+            DisplayString(screen->getDisplay()->getDisplay()));
           // gotta love pointer math
           sprintf(displaystring + dslen - 1, "%d", screen->getScreenNumber());
-	  sprintf(command, "DISPLAY=%s exec %s &", displaystring,
+	  sprintf(command, "DISPLAY=\"%s\" exec %s &", displaystring,
 		  item->exec());
 	  system(command);
-	  
+
           delete [] displaystring;
           delete [] command;
-#else
+#else // !__EMX__
 	  spawnlp(P_NOWAIT, "cmd.exe", "cmd.exe", "/c", item->exec(), NULL);
-#endif
+#endif // __EMX__
 	}
 	
 	break;
-      
-      case Blackbox::B_ExecReconfigure:
-	if (item->exec()) {
-	  char *command = new char[strlen(item->exec()) + 1];
-	  sprintf(command, "%s", item->exec());
-	  system(command);
-	  delete [] command;
-	}
-	
-      case Blackbox::B_Reconfigure:
+      	
+      case BScreen::Reconfigure:
 	blackbox->reconfigure();
 	break;
 	
-      case Blackbox::B_Restart:
+      case BScreen::Restart:
 	blackbox->restart();
 	break;
 	
-      case Blackbox::B_RestartOther:
+      case BScreen::RestartOther:
 	if (item->exec())
 	  blackbox->restart(item->exec());
 
 	break;
 	
-      case Blackbox::B_Exit:
-	blackbox->exit();
+      case BScreen::Exit:
+	blackbox->shutdown();
 	break;
 
-      case Blackbox::B_SetStyle:
+      case BScreen::SetStyle:
 	if (item->exec()) {
 	  blackbox->saveStyleFilename(item->exec());
 	  blackbox->reconfigure();
@@ -118,18 +110,11 @@ void Rootmenu::itemSelected(int button, int index) {
       }
       
       if (! (screen->getRootmenu()->hasUserMoved() || hasUserMoved()) &&
-	  item->function() != Blackbox::B_Reconfigure &&
-	  item->function() != Blackbox::B_ExecReconfigure &&
-	  item->function() != Blackbox::B_SetStyle)
+	  item->function() != BScreen::Reconfigure &&
+	  item->function() != BScreen::SetStyle)
 	screen->getRootmenu()->hide();
     }
   } else if (button == 3)
     screen->getRootmenu()->hide();
 }
 
-
-void Rootmenu::show(void) {
-  XRaiseWindow(screen->getDisplay(), getWindowID());
-
-  Basemenu::show();
-}
