@@ -1,6 +1,5 @@
-//
-// workspace.cc for Blackbox - an X11 Window manager
-// Copyright (c) 1997, 1998 by Brad Hughes, bhughes@tcac.net
+// Workspace.cc for Blackbox - an X11 Window manager
+// Copyright (c) 1997 - 1999 by Brad Hughes, bhughes@tcac.net
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,13 +18,13 @@
 // (See the included file COPYING / GPL-2.0)
 //
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
+#ifndef   _GNU_SOURCE
+#define   _GNU_SOURCE
+#endif // _GNU_SOURCE
 
-#ifdef HAVE_CONFIG_H
+#ifdef    HAVE_CONFIG_H
 #  include "../config.h"
-#endif
+#endif // HAVE_CONFIG_H
 
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
@@ -38,13 +37,16 @@
 #include "Workspace.hh"
 #include "Windowmenu.hh"
 
-#if HAVE_STDIO_H
+#ifdef    HAVE_STDIO_H
 #  include <stdio.h>
-#endif
+#endif // HAVE_STDIO_H
 
-#if STDC_HEADERS
+#ifdef    STDC_HEADERS
 #  include <string.h>
-#endif
+#endif // STDC_HEADERS
+
+#define MIN(x,y) ((x < y) ? x : y)
+#define MAX(x,y) ((x > y) ? x : y)
 
 
 Workspace::Workspace(BScreen *scrn, int i) {
@@ -56,15 +58,13 @@ Workspace::Workspace(BScreen *scrn, int i) {
 
   windowList = new LinkedList<BlackboxWindow>;
   clientmenu = new Clientmenu(screen->getBlackbox(), this);
-  clientmenu->update();
 
 #ifdef    KDE
-  {
-    char t[1024];
-    sprintf(t, "KWM_DESKTOP_NAME_%d", id + 1);
-    
-    desktop_name_atom = XInternAtom(screen->getDisplay(), t, False);
-  }
+  char t[1024];
+  sprintf(t, "KWM_DESKTOP_NAME_%d", id + 1);
+  
+  desktop_name_atom =
+    XInternAtom(screen->getDisplay()->getDisplay(), t, False);
 #endif // KDE
   
   char *tmp;  
@@ -73,8 +73,6 @@ Workspace::Workspace(BScreen *scrn, int i) {
   setName(tmp);
   
   if (tmp) delete [] tmp;
-  
-  label = 0;
 }
 
 
@@ -104,17 +102,16 @@ const int Workspace::addWindow(BlackboxWindow *w, Bool place) {
 
 
 const int Workspace::removeWindow(BlackboxWindow *w) {
-  int i = 0;
-  
   windowList->remove((const int) w->getWindowNumber());
   
+  if (clientmenu->getHighlight() == w->getWindowNumber())
+    setFocusWindow(-1);
+
   clientmenu->remove(w->getWindowNumber());
   clientmenu->update();
 
-  if (! clientmenu->getCount()) clientmenu->hide();
-
   LinkedListIterator<BlackboxWindow> it(windowList);
-  for (i = 0; it.current(); it++, i++)
+  for (int i = 0; it.current(); it++, i++)
     it.current()->setWindowNumber(i);
   
   return windowList->count();
@@ -122,13 +119,44 @@ const int Workspace::removeWindow(BlackboxWindow *w) {
 
 
 void Workspace::setFocusWindow(int w) {
-  if (w >= 0 && w < windowList->count())
-    label = getWindow(w)->getTitle();
-  else
-    label = 0;
+  if (w == clientmenu->getHighlight() &&
+      (! screen->getBlackbox()->getFocusedWindow())) return;
 
-  clientmenu->setHighlight(w);  
-  screen->getToolbar()->redrawWindowLabel(True);
+  BlackboxWindow *foc = screen->getBlackbox()->getFocusedWindow(),
+    *win = (BlackboxWindow *) 0;
+
+  if (w >= 0 && w < windowList->count())
+    win = getWindow(w);
+
+  if (foc == win) return;
+
+  if (foc && foc->validateClient()) {
+    if (foc->getWorkspaceNumber() != id)
+      screen->getWorkspace(foc->getWorkspaceNumber())->
+        clientmenu->setHighlight(-1);
+
+    foc->setFocusFlag(False);
+  }
+
+  screen->getBlackbox()->setFocusedWindow(win);
+  clientmenu->setHighlight(w);
+
+  if (id == screen->getCurrentWorkspaceID())
+    screen->getToolbar()->redrawWindowLabel(True);
+
+#ifdef    KDE
+  Blackbox *blackbox = screen->getBlackbox();
+  if (screen->getBlackbox()->getFocusedWindow() &&
+      screen->getBlackbox()->getFocusedWindow()->validateClient())
+    foc = screen->getBlackbox()->getFocusedWindow();
+
+  Window fwin = ((foc) ? foc->getClientWindow() : None);
+  XChangeProperty(blackbox->getDisplay(), screen->getRootWindow(),
+                  blackbox->getKWMWinActiveAtom(),
+                  blackbox->getKWMWinActiveAtom(), 32, PropModeReplace,
+                  ((unsigned char *) &fwin), 1);
+  screen->sendToKWMModules(blackbox->getKWMModuleWinActivateAtom(), fwin);
+#endif // KDE
 }
 
 
@@ -147,28 +175,21 @@ int Workspace::showAll(void) {
 
 
 int Workspace::hideAll(void) {
-  BlackboxWindow *win;
-
   LinkedListIterator<BlackboxWindow> it(windowList);
   for (; it.current(); it++) {
-    win = it.current();
+    BlackboxWindow *win = it.current();
     if ((! win->isIconic()) && (! win->isStuck()))
       win->withdraw();
   }
 
-  if (clientmenu->isVisible())
-    clientmenu->hide();
-
   return windowList->count();
 } 
 
 
 int Workspace::removeAll(void) {
-  BlackboxWindow *win;
-
   LinkedListIterator<BlackboxWindow> it(windowList);
   for (; it.current(); it++) {
-    win = it.current();
+    BlackboxWindow *win = it.current();
     screen->getCurrentWorkspace()->addWindow(win);
     if (! win->isIconic())
       win->iconify();
@@ -198,10 +219,21 @@ void Workspace::raiseWindow(BlackboxWindow *w) {
   win = bottom;
   while (win->hasTransient() && win->getTransient()) {
     *(nstack + (i++)) = win->getFrameWindow();
+
+#ifdef    KDE
+    screen->sendToKWMModules(screen->getBlackbox()->getKWMModuleWinRaiseAtom(),
+                             win->getFrameWindow());
+#endif // KDE
+
     win = win->getTransient();
   }
   
   *(nstack + (i++)) = win->getFrameWindow();
+
+#ifdef    KDE
+  screen->sendToKWMModules(screen->getBlackbox()->getKWMModuleWinRaiseAtom(),
+                           win->getClientWindow());
+#endif // KDE
   
   screen->raiseWindows(nstack, i);
   
@@ -222,22 +254,32 @@ void Workspace::lowerWindow(BlackboxWindow *w) {
     
     i++;
   }
-  
+ 
   Window *nstack = new Window[i];
-  
+ 
   i = 0;
-  win = bottom;
-  while (win->hasTransient() && win->getTransient()) {
+  while (win->getTransientFor()) {
     *(nstack + (i++)) = win->getFrameWindow();
-    win = win->getTransient();
+
+#ifdef    KDE
+    screen->sendToKWMModules(screen->getBlackbox()->getKWMModuleWinLowerAtom(),
+                             win->getFrameWindow());
+#endif // KDE
+
+    win = win->getTransientFor();
   }
   
   *(nstack + (i++)) = win->getFrameWindow();
+
+#ifdef    KDE
+  screen->sendToKWMModules(screen->getBlackbox()->getKWMModuleWinLowerAtom(),
+                           win->getClientWindow());
+#endif // KDE
   
   screen->getBlackbox()->grab();
   
-  XLowerWindow(screen->getDisplay(), *nstack);
-  XRestackWindows(screen->getDisplay(), nstack, i);
+  XLowerWindow(screen->getDisplay()->getDisplay(), *nstack);
+  XRestackWindows(screen->getDisplay()->getDisplay(), nstack, i);
   
   screen->getBlackbox()->ungrab();
   
@@ -290,21 +332,24 @@ void Workspace::setName(char *new_name) {
 
     int len = strlen(new_name) + 1;
     name = new char[len];
-    strncpy(name, new_name, len);
+    sprintf(name, "%s", new_name);
   } else {
     if (name) delete [] name;
     
     name = new char[32];
     if (name)
-      sprintf(name, "Workspace %d", id);
+      sprintf(name, "Workspace %d", id + 1);
   }
 
+  clientmenu->setLabel(name);
+  clientmenu->update();
+
 #ifdef    KDE
   if (desktop_name_atom) {
-    XChangeProperty(screen->getDisplay(), screen->getRootWindow(),
+    XChangeProperty(screen->getDisplay()->getDisplay(), screen->getRootWindow(),
                     desktop_name_atom, XA_STRING, 8, PropModeReplace,
                     (unsigned char *) name, strlen(name) + 1);
-
+    
     screen->sendToKWMModules(screen->getBlackbox()->
 			     getKWMModuleDesktopNameChangeAtom(), (XID) id);
   }
@@ -313,9 +358,10 @@ void Workspace::setName(char *new_name) {
 
 
 void Workspace::shutdown(void) {
-  LinkedListIterator<BlackboxWindow> it(windowList);
-  for (; it.current(); it++)
-    it.current()->restore();
+  while (windowList->count()) {
+    windowList->first()->restore();
+    delete windowList->first();
+  }
 }
 
 
@@ -331,11 +377,9 @@ void Workspace::placeWindow(BlackboxWindow *win) {
         extra = screen->getBevelWidth() * 2;
       
 #ifdef    KDE
-      {
-	int junk;
-	
-	getKWMWindowRegion(&x_origin, &y_origin, &junk, &junk);
-      }
+      int junk;
+      
+      getKWMWindowRegion(&x_origin, &y_origin, &junk, &junk);
 #endif // KDE
       
       test_x = x_origin + screen->getBevelWidth();
@@ -344,12 +388,12 @@ void Workspace::placeWindow(BlackboxWindow *win) {
       // adaptation from Window Maker's smart window placement
       
       while (((test_y + win->getHeight()) <
-	      (unsigned) (screen->getYRes() -
+	      (unsigned) (screen->getHeight() -
 			  screen->getToolbar()->getHeight() - 1)) &&
 	     (! done)) {
 	test_x = x_origin + screen->getBevelWidth();
 	
-	while (((test_x + win->getWidth()) < (unsigned) screen->getXRes()) &&
+	while (((test_x + win->getWidth()) < (unsigned) screen->getWidth()) &&
 	       (! done)) {
 	  done = True;
 	  
@@ -411,19 +455,17 @@ void Workspace::placeWindow(BlackboxWindow *win) {
     
   case BScreen::CascadePlacement:
   default:
-    if (((unsigned) cascade_x > (screen->getXRes() / 2)) ||
-	((unsigned) cascade_y > (screen->getYRes() / 2)))
+    if (((unsigned) cascade_x > (screen->getWidth() / 2)) ||
+	((unsigned) cascade_y > (screen->getHeight() / 2)))
       cascade_x = cascade_y = 32;
     
 
 #ifdef    KDE
-    {
-      int x_origin = 0, y_origin = 0, junk = 0;
-      getKWMWindowRegion(&x_origin, &y_origin, &junk, &junk);
-      
-      place_x = cascade_x + x_origin;
-      place_y = cascade_y + y_origin;
-    }
+    int x_origin = 0, y_origin = 0, junk = 0;
+    getKWMWindowRegion(&x_origin, &y_origin, &junk, &junk);
+    
+    place_x = cascade_x + x_origin;
+    place_y = cascade_y + y_origin;
 #else  // KDE
     place_x = cascade_x;
     place_y = cascade_y;
@@ -435,11 +477,11 @@ void Workspace::placeWindow(BlackboxWindow *win) {
     break;
   }
   
-  if (place_x + win->getWidth() > screen->getXRes())
-    place_x = (screen->getXRes() - win->getWidth()) / 2;
+  if (place_x + win->getWidth() > screen->getWidth())
+    place_x = (screen->getWidth() - win->getWidth()) / 2;
   if (place_y + win->getHeight() >
-      (screen->getYRes() - screen->getToolbar()->getHeight() - 1))
-    place_y = ((screen->getYRes() - screen->getToolbar()->getHeight() - 1) -
+      (screen->getHeight() - screen->getToolbar()->getHeight() - 1))
+    place_y = ((screen->getHeight() - screen->getToolbar()->getHeight() - 1) -
 	       win->getHeight()) / 2;
   
   win->configure(place_x, place_y, win->getWidth(), win->getHeight());
@@ -447,14 +489,14 @@ void Workspace::placeWindow(BlackboxWindow *win) {
 
 
 #ifdef    KDE
-
 void Workspace::getKWMWindowRegion(int *x1, int *y1, int *x2, int *y2) {
   Atom ajunk;
 
   int ijunk;
   unsigned long uljunk, *data = (unsigned long *) 0;
 
-  if (XGetWindowProperty(screen->getDisplay(), screen->getRootWindow(),
+  if (XGetWindowProperty(screen->getDisplay()->getDisplay(),
+                         screen->getRootWindow(),
                          screen->getBlackbox()->getKWMWindowRegion1Atom(),
                          0l, 4l, False,
                          screen->getBlackbox()->getKWMWindowRegion1Atom(),
@@ -477,7 +519,8 @@ void Workspace::rereadName(void) {
   int ijunk;
   unsigned long uljunk;
   
-  if (XGetWindowProperty(screen->getDisplay(), screen->getRootWindow(),
+  if (XGetWindowProperty(screen->getDisplay()->getDisplay(),
+                         screen->getRootWindow(),
 			 desktop_name_atom, 0l, ~0l, False, XA_STRING,
 			 &ajunk, &ijunk, &uljunk, &uljunk,
 			 (unsigned char **) &new_name) == Success &&
@@ -490,5 +533,4 @@ void Workspace::rereadName(void) {
     XFree((char *) new_name);
   }
 }
-
 #endif // KDE
