@@ -1,6 +1,5 @@
-//
 // main.cc for Blackbox - an X11 Window manager
-// Copyright (c) 1997, 1998 by Brad Hughes, bhughes@tcac.net
+// Copyright (c) 1997 - 1999 by Brad Hughes, bhughes@tcac.net
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,53 +18,67 @@
 // (See the included file COPYING / GPL-2.0)
 //
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
+#ifndef   _GNU_SOURCE
+#define   _GNU_SOURCE
+#endif // _GNU_SOURCE
 
-#ifdef HAVE_CONFIG_H
+#include "../version.h"
+
+#ifdef    HAVE_CONFIG_H
 #  include "../config.h"
-#endif
+#endif // HAVE_CONFIG_H
 
 #include "blackbox.hh"
 
-#ifdef HAVE_STDIO_H
+#ifdef    HAVE_STDIO_H
 #  include <stdio.h>
-#endif
+#endif // HAVE_STDIO_H
 
-#ifdef STDC_HEADERS
+#ifdef    STDC_HEADERS
 #  include <stdlib.h>
-#endif
+#endif // STDC_HEADERS
 
-#ifdef HAVE_LOCALE_H
+#ifdef    HAVE_LOCALE_H
 #  include <locale.h>
-#endif
+#endif // HAVE_LOCALE_H
 
-#ifdef HAVE_UNISTD_H
+#ifdef    HAVE_UNISTD_H
 #include <sys/types.h>
-#endif
+#endif // HAVE_UNISTD_H
 
-#ifdef HAVE_SYS_PARAM_H
+#ifdef    HAVE_SYS_PARAM_H
 #  include <sys/param.h>
-#endif
+#endif // HAVE_SYS_PARAM_H
 
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 255
-#endif
+#ifndef   MAXPATHLEN
+#define   MAXPATHLEN 255
+#endif // MAXPATHLEN
 
 
 int main(int argc, char **argv) {
-  char *session_display = NULL;
-
-#ifdef HAVE_SETLOCALE
+  char *session_display = (char *) 0;
+  char *rc_file = (char *) 0;
+ 
+#ifdef    HAVE_SETLOCALE
   setlocale(LC_ALL, "");
-#endif
+#endif // HAVE_SETLOCALE
 
   int i;
   for (i = 1; i < argc; ++i) {
-    // check for -display option... to run on a display other than the one
-    // set by the environment variable DISPLAY
-    if (! strcmp(argv[i], "-display")) {
+    if (! strcmp(argv[i], "-rc")) {
+      // look for alternative rc file to use
+
+      if ((++i) >= argc) {
+        fprintf(stderr, "error: '-rc' requires and argument\n");
+
+        ::exit(1);
+      }
+
+      rc_file = argv[i];
+    } else if (! strcmp(argv[i], "-display")) {
+      // check for -display option... to run on a display other than the one
+      // set by the environment variable DISPLAY
+
       if ((++i) >= argc) {
 	fprintf(stderr, "error: '-display' requires an argument\n");
 
@@ -83,14 +96,15 @@ int main(int argc, char **argv) {
       }
     } else if (! strcmp(argv[i], "-version")) {
       // print current version string
-      printf("Blackbox %s : (c) 1997, 1998 Brad Hughes\n\n",
+      printf("Blackbox %s : (c) 1997 - 1999 Brad Hughes\n\n",
              __blackbox_version);
 
       ::exit(0);
     } else if (! strcmp(argv[i], "-help")) {
       // print program usage and command line options
-      printf("Blackbox %s : (c) 1997, 1998 Brad Hughes\n\n"
+      printf("Blackbox %s : (c) 1997 - 1999 Brad Hughes\n\n"
              "  -display <string>\tuse display connection.\n"
+             "  -rc <string>\t\tuse alternate resource file.\n"
 	     "  -version\t\tdisplay version and exit.\n"
              "  -help\t\t\tdisplay this help text and exit.\n\n",
 	     __blackbox_version);
@@ -99,12 +113,12 @@ int main(int argc, char **argv) {
     }
   }
 
-#ifdef __EMX__
+#ifdef    __EMX__
   _chdir2(getenv("X11ROOT"));
-#endif
+#endif // __EMX__
 
-  Blackbox blackbox(argc, argv, session_display);
+  Blackbox blackbox(argc, argv, session_display, rc_file);
   blackbox.eventLoop();
-
+  
   return(0);
 }
